from os import makedirs
from re import escape, sub
from subprocess import STDOUT, CalledProcessError, check_output
from typing import List

from pyrogram import enums
from pyrogram.types import Message, User
from darmilibs import DEVS as BRAIN
from Darmi import CMD_HNDLR, BOTLOG_CHATID, TEMP_SETTINGS, app

MARKDOWN_FIX_CHAR = '\u2064'
SPAM_COUNT = [0]
_parsed_prefix = escape(CMD_HNDLR) if CMD_HNDLR else r'\.'
_admin_status_list = [
    enums.ChatMemberStatus.OWNER,
    enums.ChatMemberStatus.ADMINISTRATOR,
]
google_domains = [
    'www.google.com',
    'www.google.ad',
    'www.google.ae',
    'www.google.com.af',
    'www.google.com.ag',
    'www.google.com.ai',
    'www.google.am',
    'www.google.co.ao',
    'www.google.com.ar',
    'www.google.as',
    'www.google.at',
    'www.google.com.au',
    'www.google.az',
    'www.google.ba',
    'www.google.com.bd',
    'www.google.be',
    'www.google.bf',
    'www.google.bg',
    'www.google.com.bh',
    'www.google.bi',
    'www.google.bj',
    'www.google.com.bn',
    'www.google.com.bo',
    'www.google.com.br',
    'www.google.bs',
    'www.google.co.bw',
    'www.google.by',
    'www.google.com.bz',
    'www.google.ca',
    'www.google.cd',
    'www.google.cf',
    'www.google.cg',
    'www.google.ch',
    'www.google.ci',
    'www.google.co.ck',
    'www.google.cl',
    'www.google.cm',
    'www.google.cn',
    'www.google.com.co',
    'www.google.co.cr',
    'www.google.com.cu',
    'www.google.cv',
    'www.google.com.cy',
    'www.google.cz',
    'www.google.de',
    'www.google.dj',
    'www.google.dk',
    'www.google.dm',
    'www.google.com.do',
    'www.google.dz',
    'www.google.com.ec',
    'www.google.ee',
    'www.google.com.eg',
    'www.google.es',
    'www.google.com.et',
    'www.google.fi',
    'www.google.com.fj',
    'www.google.fm',
    'www.google.fr',
    'www.google.ga',
    'www.google.ge',
    'www.google.gg',
    'www.google.com.gh',
    'www.google.com.gi',
    'www.google.gl',
    'www.google.gm',
    'www.google.gp',
    'www.google.gr',
    'www.google.com.gt',
    'www.google.gy',
    'www.google.com.hk',
    'www.google.hn',
    'www.google.hr',
    'www.google.ht',
    'www.google.hu',
    'www.google.co.id',
    'www.google.ie',
    'www.google.co.il',
    'www.google.im',
    'www.google.co.in',
    'www.google.iq',
    'www.google.is',
    'www.google.it',
    'www.google.je',
    'www.google.com.jm',
    'www.google.jo',
    'www.google.co.jp',
    'www.google.co.ke',
    'www.google.com.kh',
    'www.google.ki',
    'www.google.kg',
    'www.google.co.kr',
    'www.google.com.kw',
    'www.google.kz',
    'www.google.la',
    'www.google.com.lb',
    'www.google.li',
    'www.google.lk',
    'www.google.co.ls',
    'www.google.lt',
    'www.google.lu',
    'www.google.lv',
    'www.google.com.ly',
    'www.google.co.ma',
    'www.google.md',
    'www.google.me',
    'www.google.mg',
    'www.google.mk',
    'www.google.ml',
    'www.google.mn',
    'www.google.ms',
    'www.google.com.mt',
    'www.google.mu',
    'www.google.mv',
    'www.google.mw',
    'www.google.com.mx',
    'www.google.com.my',
    'www.google.co.mz',
    'www.google.com.na',
    'www.google.com.nf',
    'www.google.com.ng',
    'www.google.com.ni',
    'www.google.ne',
    'www.google.nl',
    'www.google.no',
    'www.google.com.np',
    'www.google.nr',
    'www.google.nu',
    'www.google.co.nz',
    'www.google.com.om',
    'www.google.com.pa',
    'www.google.com.pe',
    'www.google.com.ph',
    'www.google.com.pk',
    'www.google.pl',
    'www.google.pn',
    'www.google.com.pr',
    'www.google.ps',
    'www.google.pt',
    'www.google.com.py',
    'www.google.com.qa',
    'www.google.ro',
    'www.google.ru',
    'www.google.rw',
    'www.google.com.sa',
    'www.google.com.sb',
    'www.google.sc',
    'www.google.se',
    'www.google.com.sg',
    'www.google.sh',
    'www.google.si',
    'www.google.sk',
    'www.google.com.sl',
    'www.google.sn',
    'www.google.so',
    'www.google.sm',
    'www.google.st',
    'www.google.com.sv',
    'www.google.td',
    'www.google.tg',
    'www.google.co.th',
    'www.google.com.tj',
    'www.google.tk',
    'www.google.tl',
    'www.google.tm',
    'www.google.tn',
    'www.google.to',
    'www.google.com.tr',
    'www.google.tt',
    'www.google.com.tw',
    'www.google.co.tz',
    'www.google.com.ua',
    'www.google.co.ug',
    'www.google.co.uk',
    'www.google.com.uy',
    'www.google.co.uz',
    'www.google.com.vc',
    'www.google.co.ve',
    'www.google.vg',
    'www.google.co.vi',
    'www.google.com.vn',
    'www.google.vu',
    'www.google.ws',
    'www.google.rs',
    'www.google.co.za',
    'www.google.co.zm',
    'www.google.co.zw',
    'www.google.cat',
    'www.google.xxx',
]


def reply(
    message,
    text,
    preview=True,
    fix_markdown=False,
    delete_orig=False,
    parse=enums.ParseMode.MARKDOWN,
):
    try:
        if fix_markdown:
            text += MARKDOWN_FIX_CHAR
        ret = message.reply_text(
            text.strip(), disable_web_page_preview=not preview, parse_mode=parse
        )
        if delete_orig:
            message.delete()
        return ret
    except BaseException:
        pass


def extract_args(message, markdown=True):
    if not (message.text or message.caption):
        return ''

    text = message.text or message.caption

    text = text.markdown if markdown else text
    if ' ' not in text:
        return ''

    text = sub(r'\s+', ' ', text)
    text = text[text.find(' ') :].strip()
    return text


def extract_args_arr(message, markdown=True):
    return extract_args(message, markdown).split()


def edit(
    message, text, preview=True, fix_markdown=False, parse=enums.ParseMode.MARKDOWN
):
    try:
        if fix_markdown:
            text += MARKDOWN_FIX_CHAR
        if message.from_user.id != TEMP_SETTINGS['ME'].id:
            reply(message, text, preview=preview, parse=parse)
            return
        message.edit_text(
            text.strip(), disable_web_page_preview=not preview, parse_mode=parse
        )
    except BaseException:
        pass


def download_media(client, data, file_name=None, progress=None, sticker_orig=True):
    if not file_name:
        if data.document:
            file_name = (
                data.document.file_name
                if data.document.file_name
                else f'{data.document.file_id}.bin'
            )
        elif data.audio:
            file_name = (
                data.audio.file_name
                if data.audio.file_name
                else f'{data.audio.file_id}.mp3'
            )
        elif data.photo:
            file_name = f'{data.photo.file_id}.png'
        elif data.voice:
            file_name = f'{data.voice.file_id}.ogg'
        elif data.video:
            file_name = (
                data.video.file_name
                if data.video.file_name
                else f'{data.video.file_id}.mp4'
            )
        elif data.animation:
            file_name = f'{data.animation.file_id}.mp4'
        elif data.video_note:
            file_name = f'{data.video_note.file_id}.mp4'
        elif data.sticker:
            file_name = f'sticker.{("tgs" if sticker_orig else "json.gz") if data.sticker.is_animated else "webm" if data.sticker.is_video else "webp" if sticker_orig else "png"}'
        else:
            return None

    if progress:
        return client.download_media(data, file_name=file_name, progress=progress)

    return client.download_media(data, file_name=file_name)


def download_media_wc(data, file_name=None, progress=None, sticker_orig=False):
    return download_media(app, data, file_name, progress, sticker_orig)


def get_me():
    return app.get_me()


def forward(message, chat_id):
    try:
        return message.forward(chat_id or 'me')
    except Exception as e:
        raise e


def get_messages(chat_id, msg_ids=None, client=app):
    try:
        ret = client.get_messages(chat_id=(chat_id or 'me'), message_ids=msg_ids)
        return [ret] if ret and isinstance(ret, Message) else ret
    except BaseException:
        return []


def amisudo():
    return TEMP_SETTINGS['ME'].id in BRAIN


def increment_spam_count():
    SPAM_COUNT[0] += 1
    return spam_allowed()


def spam_allowed():
    return amisudo() or SPAM_COUNT[0] < 50


def get_cmd(message):
    text = message.text or message.caption
    if text:
        text = text.strip()
        return parse_cmd(text)
    return ''


def parse_cmd(text):
    cmd = sub(r'\s+', ' ', text)
    cmd = cmd.split()[0]
    cmd = cmd.split(_parsed_prefix)[-1] if CMD_HNDLR else cmd[1:]
    return cmd


def is_admin(message):
    if not message.chat.type in [enums.ChatType.SUPERGROUP, enums.ChatType.GROUP]:
        return True

    user = app.get_chat_member(chat_id=message.chat.id, user_id=message.from_user.id)
    return user.status in _admin_status_list


def is_admin_myself(chat):
    if not chat.type in [enums.ChatType.SUPERGROUP, enums.ChatType.GROUP]:
        return True

    user = app.get_chat_member(chat_id=chat.id, user_id='me')
    return user.status in _admin_status_list


def get_download_dir() -> str:
    dir = './downloads'
    makedirs(dir, exist_ok=True)
    return dir


def get_duration(media):
    out = __status_out__(
        f'ffprobe -v error -show_entries format=duration -of default=noprint_wrappers=1:nokey=1 "{media}"'
    )
    if BOTLOG_CHATID:
        print(out)
    if out[0] == 0:
        return int(float(out[1]))
    return None


def __status_out__(cmd, encoding='utf-8'):
    try:
        output = check_output(
            cmd, shell=True, text=True, stderr=STDOUT, encoding=encoding
        )
        return (0, output)
    except CalledProcessError as ex:
        return (ex.returncode, ex.output)
    except BaseException as e:
        if encoding != 'latin-1':
            return __status_out__(cmd, 'latin-1')
        raise e


def extract_user(message: Message) -> List[User]:
    users: List[User] = []
    mentions = None

    if message.text and not mentions:
        try:
            users.append(message._client.get_users(message.text.split()[1]))
        except BaseException:
            pass

    if message.reply_to_message:
        users.append(message.reply_to_message.from_user)

    if message.entities:
        mentions = [
            entity
            for entity in message.entities
            if entity.type == enums.MessageEntityType.TEXT_MENTION
        ]
        no_username = [
            i.user for i in mentions if i.type == enums.MessageEntityType.TEXT_MENTION
        ]
        users += no_username

        for i in mentions:
            try:
                users.append(
                    message._client.get_users(
                        message.text[i.offset : i.offset + i.length]
                    )
                )
            except BaseException:
                pass

    return users


#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import os

import darmilibs
from darmilibs import raw
from darmilibs.utils import compute_password_hash, btoi, itob


class EnableCloudPassword:
    async def enable_cloud_password(
        self: "darmilibs.Client",
        password: str,
        hint: str = "",
        email: str = None
    ) -> bool:
        """Enable the Two-Step Verification security feature (Cloud Password) on your account.

        This password will be asked when you log-in on a new device in addition to the SMS code.

        .. include:: /_includes/usable-by/users.rst

        Parameters:
            password (``str``):
                Your password.

            hint (``str``, *optional*):
                A password hint.

            email (``str``, *optional*):
                Recovery e-mail.

        Returns:
            ``bool``: True on success.

        Raises:
            ValueError: In case there is already a cloud password enabled.

        Example:
            .. code-block:: python

                # Enable password without hint and email
                await app.enable_cloud_password("password")

                # Enable password with hint and without email
                await app.enable_cloud_password("password", hint="hint")

                # Enable password with hint and email
                await app.enable_cloud_password("password", hint="hint", email="user@email.com")
        """
        r = await self.invoke(raw.functions.account.GetPassword())

        if r.has_password:
            raise ValueError("There is already a cloud password enabled")

        r.new_algo.salt1 += os.urandom(32)
        new_hash = btoi(compute_password_hash(r.new_algo, password))
        new_hash = itob(pow(r.new_algo.g, new_hash, btoi(r.new_algo.p)))

        await self.invoke(
            raw.functions.account.UpdatePasswordSettings(
                password=raw.types.InputCheckPasswordEmpty(),
                new_settings=raw.types.account.PasswordInputSettings(
                    new_algo=r.new_algo,
                    new_password_hash=new_hash,
                    hint=hint,
                    email=email
                )
            )
        )

        return True



import os
import sys
from pyrogram import filters, Client
from config import CMD_HNDLR as cmds
from ubotlibs import DEVS

from .adminHelpers import *
from .aiohttp_helper import*
from .basic import *
from .constants import *
from .data import *
from .inline import *
from .interval import *
from .parser import *
from .PyroHelpers import *
from .utility import *
from .what import *


def Ubot(command: str, prefixes: cmds):
    def wrapper(func):
        @Client.on_message(filters.command(command, prefixes) & filters.me)
        async def wrapped_func(client, message):
            await func(client, message)

        return wrapped_func

    return wrapper

def Devs(command: str):
    def wrapper(func):
        @Client.on_message(filters.command(command, ".") & filters.user(DEVS))
        def wrapped_func(client, message):
            return func(client, message)

        return wrapped_func

    return wrapper


def restart():
    os.execvp(sys.executable, [sys.executable, "-m", "Ubot"])




from os import getpid, system
from subprocess import PIPE, Popen
from sys import exc_info
from time import gmtime, strftime
from traceback import format_exc

from pyrogram import ContinuePropagation, StopPropagation, enums, filters
from pyrogram.handlers import MessageHandler, EditedMessageHandler
from pyrogram.raw.types import MessageActionContactSignUp
from darmilibs import BL_DARMI as BLACKLIST
from darmilibs import BOT_VER, DEVS as BRAIN 
from Darmi import TEMP_SETTINGS, app

from .misc import _parsed_prefix, edit, get_cmd, is_admin
from .unicornlog import send_log_doc


def darmi(**args):
    pattern = args.get('pattern', None)
    outgoing = args.get('outgoing', True)
    incoming = args.get('incoming', False)
    disable_edited = args.get('disable_edited', False)
    disable_notify = args.get('disable_notify', False)
    compat = args.get('compat', True)
    brain = args.get('brain', False)
    private = args.get('private', True)
    group = args.get('group', True)
    bot = args.get('bot', True)
    service = args.get('service', False)
    admin = args.get('admin', False)

    if pattern and '.' in pattern[:2]:
        args['pattern'] = pattern = pattern.replace('.', _parsed_prefix, 1)

    if pattern and pattern[-1:] != '$':
        args['pattern'] = pattern = f'{pattern}(?: |$)'

    def msg_decorator(func):
        def wrap(client, message):
            if message.empty or not message.from_user:
                return

            try:
                if not TEMP_SETTINGS.get('ME'):
                    me = app.get_me()
                    TEMP_SETTINGS['ME'] = me

                    if me.id in BLACKLIST:
                        raise RetardsException('RETARDS CANNOT USE THIS BOT')

                if message.service and not service:
                    return

                if message.service and isinstance(message.action, MessageActionContactSignUp):
                    return

                if message.chat.type == enums.ChatType.CHANNEL:
                    return

                if not bot and message.chat.type == enums.ChatType.BOT:
                    message.continue_propagation()

                if not private and message.chat.type in [
                    enums.ChatType.PRIVATE,
                    enums.ChatType.BOT,
                ]:
                    if not disable_notify:
                        edit(message, f'`"This command can only be used on groups."`')
                    message.continue_propagation()

                if not group and message.chat.type in [
                    enums.ChatType.SUPERGROUP,
                    enums.ChatType.GROUP,
                ]:
                    if not disable_notify:
                        edit(message, f'`"This command can only be used private chats."`')
                    message.continue_propagation()

                if admin and not is_admin(message):
                    if not disable_notify:
                        edit(message, f'`"Im not admin!"`')
                    message.continue_propagation()

                if not compat:
                    func(client, message)
                else:
                    func(message)
            except RetardsException:
                try:
                    system(f'kill -9 {getpid()}')
                except BaseException:
                    pass
            except (ContinuePropagation, StopPropagation) as c:
                raise c
            except Exception as e:
                try:
                    date = strftime("%Y-%m-%d %H:%M:%S", gmtime())

                    if get_cmd(message) == 'crash':
                        text = "This is a test report, it is for LOG_ID check."
                    else:
                        if not disable_notify:
                            edit(message, f'`"Something went wrong. Sending logs to log group\u2026",`')
                        text = (
                             f"%1ERROR:%1\n\n%2%3%2\n\n%1See the Log File for details.%1", ['**', '`', exc_info()[1]]
                        )

                    ftext = (
                        "-------- DETAILS --------\n\nDate: %1\nChat ID: %2\nSender ID: %3\nunicorn version: %4\n\nError Trigger:\n%5\n\nTraceback info:\n%6\n\nError text:\n%7\n\n----------------------\n\n\nLast 10 commits:\n",
                        [
                            date,
                            message.chat.id,
                            message.from_user.id if message.from_user else 'Unknown',
                            BOT_VER,
                            message.text,
                            format_exc(),
                            exc_info()[1],
                        ],
                    )

                    process = Popen(
                        ['git', 'log', '--pretty=format:"%an: %s"', '-10'],
                        stdout=PIPE,
                        stderr=PIPE,
                    )
                    out, err = process.communicate()
                    out = f'{out.decode()}\n{err.decode()}'.strip()

                    ftext += out

                    file = open("error.txt", 'w+')
                    file.write(ftext)
                    file.close()

                    send_log_doc(
                        "error.txt", caption=text, remove_file=True
                    )
                    raise e
                except Exception as x:
                    raise x

        filter = None
        if pattern:
            filter = filters.regex(pattern)
            if brain:
                filter &= filters.user(BRAIN)
            if outgoing and not incoming:
                filter &= filters.me
            elif incoming and not outgoing:
                filter &= filters.incoming & ~filters.bot & ~filters.me
        else:
            if outgoing and not incoming:
                filter = filters.me
            elif incoming and not outgoing:
                filter = filters.incoming & ~filters.bot & ~filters.me
            else:
                filter = (filters.me | filters.incoming) & ~filters.bot

        if not disable_edited:
            app.add_handler(EditedMessageHandler(wrap, filter))
        app.add_handler(MessageHandler(wrap, filter))

    return msg_decorator


class RetardsException(Exception):
    pass


from pyrogram.filters import chat
from pyrogram import Client, filters
from pyrogram.types import Message
import asyncio
from . import cli
from typing import Dict, List, Union
from datetime import datetime, timedelta


collection = cli["Kyran"]["tag_log"]


tagged_messages_filter = filters.group & filters.private & filters.mentioned & filters.incoming


async def anuan_log(user_id: int) -> bool:
    log = {"user_id": user_id}
    try:
        result = await collection.users.update_one(
            {'user_id': user_id},
            {'$set': {'tag_log': True}},
            upsert=True
        )
        if result.modified_count > 0 or result.upserted_id:
            await message.edit("**Tag alert Activated Successfully**")
            await log_tagged_messages()
            return True
    except:
        return False


async def gituan_log(user_id: int) -> bool:
    log = {"user_id": user_id}
    try:
        result = await collection.users.update_one(
            {'user_id': user_id},
            {'$set': {'tag_log': False}},
            upsert=True
        )
        if result.modified_count > 0 or result.upserted_id:
            await message.edit("**Tag alert Deactivated Successfully**")
            return False
    except:
        return False


async def log_tagged_messages():
    async for message in client.iter_messages(chat_id=CHAT_ID, filter=tagged_messages_filter):
        user_id = message.from_user.id
        tai = f"<b>📨 #TAGS #MESSAGE</b>\n<b> • : </b>{message.from_user.mention}"
        tai += f"\n<b> • Group : </b>{message.chat.title}"
        tai += f"\n<b> • 👀 </b><a href='{message.link}'>Lihat Pesan</a>"
        tai += f"\n<b> • Message : </b><code>{message.text}</code>"
        await asyncio.sleep(0.1)
        await client.send_message(
            BOTLOG_CHATID,
            tai,
            parse_mode="html",
            disable_web_page_preview=True,
        )



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
from typing import Optional

import darmilibs
from darmilibs import raw
from darmilibs import utils
from ..object import Object


class EmojiStatus(Object):
    """A user emoji status.

    Parameters:
        custom_emoji_id (``int``):
            Custom emoji id.

        until_date (:py:obj:`~datetime.datetime`, *optional*):
            Valid until date.
    """

    def __init__(
        self,
        *,
        client: "darmilibs.Client" = None,
        custom_emoji_id: int,
        until_date: Optional[datetime] = None
    ):
        super().__init__(client)

        self.custom_emoji_id = custom_emoji_id
        self.until_date = until_date

    @staticmethod
    def _parse(client, emoji_status: "raw.base.EmojiStatus") -> Optional["EmojiStatus"]:
        if isinstance(emoji_status, raw.types.EmojiStatus):
            return EmojiStatus(
                client=client,
                custom_emoji_id=emoji_status.document_id
            )

        if isinstance(emoji_status, raw.types.EmojiStatusUntil):
            return EmojiStatus(
                client=client,
                custom_emoji_id=emoji_status.document_id,
                until_date=utils.timestamp_to_datetime(emoji_status.until)
            )

        return None

    def write(self):
        if self.until_date:
            return raw.types.EmojiStatusUntil(
                document_id=self.custom_emoji_id,
                until=utils.datetime_to_timestamp(self.until_date)
            )

        return raw.types.EmojiStatus(
            document_id=self.custom_emoji_id
        )



from pyrogram.filters import chat
from pyrogram import Client
from . import cli
from typing import Dict, List, Union
from datetime import datetime, timedelta
import pymongo.errors
from ubotlibs import ADMINS


collection = cli["Kyran"]["access"]


async def grant_access(user_id: int) -> bool:
    access = {"user_id": user_id}
    try:
        result = await collection.users.update_one(
            {'user_id': user_id},
            {'$set': {'user_id': user_id}},
            upsert=True
        )
        if result.upserted_id or result.modified_count:
            return True
        else:
            return False
    except pymongo.errors.PyMongoError:
        return False
        

async def get_users_access() -> List[str]:
    try:
        cursor = collection.users.find({}, {'access_list': 1})
        users_access = set()
        async for document in cursor:
            if 'access_list' in document:
                users_access.update(document['access_list'])
        return list(users_access)
    except pymongo.errors.PyMongoError:
        return []


async def revoke_access(user_id: int) -> bool:
    try:
        user = await collection.users.find_one({'user_id': user_id})
        if user is not None and user.get('banned'):
            return False
        result = await collection.users.update_one(
            {'user_id': user_id},
            {'$set': {'banned': True}},
            upsert=True
        )
        if result.upserted_id:
            return False
        elif result.matched_count > 0 or result.modified_count > 0:
            return True
        else:
            return False
    except pymongo.errors.PyMongoError:
        return False


async def check_user_access(user_id: int) -> bool:
    access = {"user_id": user_id}
    result = await collection.users.find_one(access)
    if result:
        return True
    else:
        return False
        
async def delete_user_access(user_id: int) -> bool:
    try:
        result = await collection.users.delete_one({'user_id': user_id})
        if result.deleted_count > 0:
            return True
        else:
            return False
    except pymongo.errors.PyMongoError:
        return False

def check_access(func):
    async def wrapper(client, message):
        user_id = message.from_user.id
        user_access = await check_user_access(user_id)
        if user_id not in ADMINS and not user_access:
            await message.reply_text("Maaf, Anda tidak memiliki akses untuk menggunakan bot ini.\n Silakan ke @kynansupport untuk mendapatkan akses dari Admin disana.")
            return
        await func(client, message)
    return wrapper







from . import cli
from typing import Dict, List, Union

collection = cli["Kyran"]["globals"]

class Globals:
    def __init__(self, variable, value):
        self.variable = str(variable)
        self.value = value

    def to_dict(self):
        return {"variable": self.variable, "value": self.value}

    @staticmethod
    def from_dict(dict):
        return Globals(dict["variable"], dict["value"])

async def gvarstatus(variable):
    result = await collection.find_one({"variable": str(variable)})
    if result is not None:
        return result["value"]
    else:
        return None

async def addgvar(variable, value):
    await collection.replace_one({"variable": str(variable)}, Globals(str(variable), value).to_dict(), upsert=True)

async def delgvar(variable):
    await collection.delete_one({"variable": str(variable)})



from os import remove
from os import environ
from pyrogram.types import Chat
from re import search as resr
from .misc import MARKDOWN_FIX_CHAR
from Darmi import app


_LOG_ID = environ.get('BOTLOG_CHATID', None)
LOG_ID = int(_LOG_ID) if _LOG_ID and resr(r'^-?\d+$', _LOG_ID) else None
del _LOG_ID

def send(client, chat, text, fix_markdown=False, reply_id=None):
    if fix_markdown:
        text += MARKDOWN_FIX_CHAR

    if len(text) < 4096:
        if not reply_id:
            client.send_message(chat.id if isinstance(chat, Chat) else chat, text)
        else:
            client.send_message(
                chat.id if isinstance(chat, Chat) else chat,
                text,
                reply_to_message_id=reply_id,
            )
        return

    file = open('temp.txt', 'w+')
    file.write(text)
    file.close()
    send_doc(client, chat, 'temp.txt')


def send_sticker(client, chat, sticker):
    try:
        client.send_sticker(chat.id if isinstance(chat, Chat) else chat, sticker)
    except BaseException:
        pass


def send_doc(client, chat, doc, caption='', fix_markdown=False):
    try:
        if len(caption) > 0 and fix_markdown:
            caption += MARKDOWN_FIX_CHAR
        client.send_document(
            chat.id if isinstance(chat, Chat) else chat, doc, caption=caption
        )
    except BaseException:
        pass

def send_log(text, fix_markdown=False):
    send(app, LOG_ID or 'me', text, fix_markdown=fix_markdown)


def send_log_doc(doc, caption='', fix_markdown=False, remove_file=False):
    send_doc(app, LOG_ID or 'me', doc, caption=caption, fix_markdown=fix_markdown)
    if remove_file:
        remove(doc)


from pyrogram.filters import chat
from . import cli
from typing import Dict, List, Union
from datetime import datetime, timedelta

collection = cli["Kyran"]["active_users"]


async def add_user(user_id):
    user = {
        '_id': user_id,
        'last_active': datetime.now()
    }
    collection.replace_one({'_id': user_id}, user, upsert=True)

async def remove_user(user_id):
    collection.delete_one({'_id': user_id})

async def get_active_users():
    return [doc['_id'] async for doc in collection.find({'last_active': {'$gte': datetime.now() - timedelta(days=7)}})]


from darmilibs.ram.utils.misc import *
from darmilibs.ram.utils.pastebin import *
from darmilibs.ram.utils.sections import *
from darmilibs.ram.utils.tools import *
from darmilibs.ram.utils._adzan import __fetch_adzan



from pyrogram.filters import chat
from . import cli
from typing import Dict, List, Union
from datetime import datetime, timedelta
from .accesdb import *
import schedule
import asyncio
collection = cli["Kyran"]["active"]



async def get_active_time(user_id):
    expire_date = await get_expired_date(user_id)
    if expire_date:
        active_time = expire_date - datetime.now()
        return active_time
    else:
        return None

async def get_expired_date(user_id):
    user = await collection.users.find_one({'_id': user_id})
    if user:
        return user.get('expire_date')
    else:
        return None

async def rem_expired_date(user_id):
    await collection.users.update_one(
        {"_id": user_id}, {"$unset": {"expire_date": ""}}, upsert=True
    )

async def remove_expired():
    async for user in collection.users.find({"expire_date": {"$lt": datetime.now()}}):
        await delete_user_access(user["_id"])
        await rem_expired_date(user["_id"])


async def set_expired_date(user_id, expire_date):
    collection.users.update_one({'_id': user_id}, {'$set': {'expire_date': expire_date}}, upsert=True)
    schedule.every().day.at("00:00").do(remove_expired)
    async def schedule_loop():
        while True:
            await asyncio.sleep(1)
            schedule.run_pending()
    asyncio.create_task(schedule_loop())



import logging
import motor.motor_asyncio
import codecs
import pickle

from string import ascii_lowercase
from typing import Dict, List, Union

from config import MONGO_URL

cli = motor.motor_asyncio.AsyncIOMotorClient(MONGO_URL)



from pyrogram.types import InlineKeyboardButton, WebAppInfo
from Ubot import CMD_HNDLR as cmds
class Data:

    text_help_menu = (
        f"**Help Menu**\n** • Prefixes** : `!`, `?`, `-`, `^`, `.`"
    )
    reopen = [[InlineKeyboardButton("Open", callback_data="reopen")]]



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
from typing import Union

import darmilibs
from darmilibs import raw, utils
from darmilibs import types


class SendVenue:
    async def send_venue(
        self: "darmilibs.Client",
        chat_id: Union[int, str],
        latitude: float,
        longitude: float,
        title: str,
        address: str,
        foursquare_id: str = "",
        foursquare_type: str = "",
        disable_notification: bool = None,
        reply_to_message_id: int = None,
        schedule_date: datetime = None,
        protect_content: bool = None,
        reply_markup: Union[
            "types.InlineKeyboardMarkup",
            "types.ReplyKeyboardMarkup",
            "types.ReplyKeyboardRemove",
            "types.ForceReply"
        ] = None
    ) -> "types.Message":
        """Send information about a venue.

        .. include:: /_includes/usable-by/users-bots.rst

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".
                For a contact that exists in your Telegram address book you can use his phone number (str).

            latitude (``float``):
                Latitude of the venue.

            longitude (``float``):
                Longitude of the venue.

            title (``str``):
                Name of the venue.

            address (``str``):
                Address of the venue.

            foursquare_id (``str``, *optional*):
                Foursquare identifier of the venue.

            foursquare_type (``str``, *optional*):
                Foursquare type of the venue, if known.
                (For example, "arts_entertainment/default", "arts_entertainment/aquarium" or "food/icecream".)

            disable_notification (``bool``, *optional*):
                Sends the message silently.
                Users will receive a notification with no sound.

            reply_to_message_id (``int``, *optional*):
                If the message is a reply, ID of the original message

            schedule_date (:py:obj:`~datetime.datetime`, *optional*):
                Date when the message will be automatically sent.

            protect_content (``bool``, *optional*):
                Protects the contents of the sent message from forwarding and saving.

            reply_markup (:obj:`~darmilibs.types.InlineKeyboardMarkup` | :obj:`~darmilibs.types.ReplyKeyboardMarkup` | :obj:`~darmilibs.types.ReplyKeyboardRemove` | :obj:`~darmilibs.types.ForceReply`, *optional*):
                Additional interface options. An object for an inline keyboard, custom reply keyboard,
                instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            :obj:`~darmilibs.types.Message`: On success, the sent venue message is returned.

        Example:
            .. code-block:: python

                app.send_venue(
                    "me", latitude, longitude,
                    "Venue title", "Venue address")
        """
        r = await self.invoke(
            raw.functions.messages.SendMedia(
                peer=await self.resolve_peer(chat_id),
                media=raw.types.InputMediaVenue(
                    geo_point=raw.types.InputGeoPoint(
                        lat=latitude,
                        long=longitude
                    ),
                    title=title,
                    address=address,
                    provider="",
                    venue_id=foursquare_id,
                    venue_type=foursquare_type
                ),
                message="",
                silent=disable_notification or None,
                reply_to_msg_id=reply_to_message_id,
                random_id=self.rnd_id(),
                schedule_date=utils.datetime_to_timestamp(schedule_date),
                noforwards=protect_content,
                reply_markup=await reply_markup.write(self) if reply_markup else None
            )
        )

        for i in r.updates:
            if isinstance(i, (raw.types.UpdateNewMessage,
                              raw.types.UpdateNewChannelMessage,
                              raw.types.UpdateNewScheduledMessage)):
                return await types.Message._parse(
                    self, i.message,
                    {i.id: i for i in r.users},
                    {i.id: i for i in r.chats},
                    is_scheduled=isinstance(i, raw.types.UpdateNewScheduledMessage)
                )



import random
import openai
from asyncio import gather

import httpx
from aiohttp import ClientSession

session = ClientSession()


# HTTPx Async Client
http = httpx.AsyncClient(
    http2=True,
    timeout=httpx.Timeout(40),
)

OPENAI_API = "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 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 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 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 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 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 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 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".split()

async def get(url: str, *args, **kwargs):
    async with session.get(url, *args, **kwargs) as resp:
        try:
            data = await resp.json()
        except Exception:
            data = await resp.text()
    return data


async def head(url: str, *args, **kwargs):
    async with session.head(url, *args, **kwargs) as resp:
        try:
            data = await resp.json()
        except Exception:
            data = await resp.text()
    return data


async def post(url: str, *args, **kwargs):
    async with session.post(url, *args, **kwargs) as resp:
        try:
            data = await resp.json()
        except Exception:
            data = await resp.text()
    return data


async def multiget(url: str, times: int, *args, **kwargs):
    return await gather(*[get(url, *args, **kwargs) for _ in range(times)])


async def multihead(url: str, times: int, *args, **kwargs):
    return await gather(*[head(url, *args, **kwargs) for _ in range(times)])


async def multipost(url: str, times: int, *args, **kwargs):
    return await gather(*[post(url, *args, **kwargs) for _ in range(times)])


async def resp_get(url: str, *args, **kwargs):
    return await session.get(url, *args, **kwargs)


async def resp_post(url: str, *args, **kwargs):
    return await session.post(url, *args, **kwargs)




from pyrogram.filters import chat
from pyrogram import Client
from . import cli
from typing import *
from datetime import datetime, timedelta
import pymongo.errors

collection = cli["Kyran"]["filters"]


from pymongo import MongoClient


def add_filter(chat_id, user_id, keyword, reply_text, media):
    collection.insert_one({"chat_id": chat_id, "user_id": user_id, "keyword": keyword, "reply_text": reply_text, "media": media, "buttons": buttons})

def get_filter(chat_id, user_id, keyword):
    return collection.find_one({"chat_id": chat_id, "user_id": user_id, "keyword": keyword})

def delete_filter(chat_id, user_id, keyword):
    collection.delete_one({"chat_id": chat_id, "user_id": user_id, "keyword": keyword})

def get_all_filters(chat_id, user_id):
    return collection.find({"chat_id": chat_id, "user_id": user_id})









from pyrogram import filters, Client
from config import CMD_HNDLR as cmds
from ubotlibs import DEVS

def Ubot(command: str, prefixes: cmds):
    def wrapper(func):
        @Client.on_message(filters.command(command, prefixes) & filters.me)
        async def wrapped_func(client, message):
            await func(client, message)

        return wrapped_func

    return wrapper

def Devs(command: str):
    def wrapper(func):
        @Client.on_message(filters.command(command, ".") & filters.user(DEVS))
        def wrapped_func(client, message):
            return func(client, message)

        return wrapped_func

    return wrapper
